@@ -3,13 +3,14 @@ import random
 import random as rand
 import numpy as np
 from numpy.lib.function_base import average
-import pygame as py
+import pygame as pg
 from pygame import pixelcopy
 from pygame.transform import flip
 from pygame.draw import line
 from pygame.version import ver
 import math as mh
 import lddya.Ipygame as ipg
+import pandas as pd
 
 ################################################## 1 蚁群算法路径规划 ###########################################
 
@@ -946,468 +947,138 @@ class PSO_min():
 
 ###################################### 99 栅格图 ######################################################################
             
-
-class ShanGeTu():
-    def __init__(self,map_path = 'map.dll',map_data = None,reverse = False):
-        self.grid_size = 0
-        self.cell_size = 0
-        self.line_size = 0
-        self.__load_map(map_path,map_data,reverse = reverse)                        #读取地图数据
-        self.pic_backgroud = self.__backgroud()  #画网格
-        self.__draw_barrier()                    #画障碍物方格，跟上一步共同组合成完整的背景图片
-        #self.draw_way(way_data=way_data)
-        #self.save()                            #保存起来
+class A_Star_path():
+    def __init__(self,map_data,start = np.array([0,0]),end = np.array([0,0])) -> None:
+        print('A* Task:',start,end)
+        self.map_data = np.array(map_data)
+        self.size_map = self.map_data.shape[0]
+        self.start = start
+        self.end   =  end
+        self.open_list  = pd.DataFrame([[start[0],start[1],0,abs(self.start[0]-self.end[0])+abs(self.start[1]-self.end[1]),abs(self.start[0]-self.end[0])+abs(self.start[1]-self.end[1]),-1,-1]],columns=['pos_y','pos_x','g','h','f','parent_y','parent_x'])
+        self.close_list = pd.DataFrame([[999,999,0,0,0,0,0]],columns=['pos_y','pos_x','g','h','f','parent_y','parent_x'])
+        self.best_way_len = 0
         
-    def __backgroud(self):
-        '''
-            Function:
-            ---------
-                绘制栅格图的背景(即不包含路径线条的栅格图),并以返回Surface形式返回
 
-            Params:
-            -------
-                None
-
-            Return:
-            -------
-                backgroud : pygame.Surface
-                    栅格图背景
-        '''
-        size = self.grid_size
-        if size == 20:
-            self.cell_size = 25
-            self.line_size = 1
-            pic_size = size*self.cell_size+(size+1)*self.line_size
-            self.backgroud_size = pic_size
-            backgroud = py.Surface([pic_size,pic_size])
-            backgroud.fill([255,255,255])
-            for i in range(size+1):
-                py.draw.line(backgroud,[0,0,0],[i*(self.cell_size+self.line_size),0],[i*(self.cell_size+self.line_size),pic_size])
-                py.draw.line(backgroud,[0,0,0],[0,i*(self.cell_size+self.line_size)],[pic_size,i*(self.cell_size+self.line_size)])
-            return backgroud
-        #elif size == 30:
-        else:
-            self.cell_size = 15
-            self.line_size = 1
-            pic_size = size*self.cell_size+(size+1)*self.line_size
-            self.backgroud_size = pic_size
-            backgroud = py.Surface([pic_size,pic_size])
-            backgroud.fill([255,255,255])
-            for i in range(size+1):
-                py.draw.line(backgroud,[0,0,0],[i*(self.cell_size+self.line_size),0],[i*(self.cell_size+self.line_size),pic_size])
-                py.draw.line(backgroud,[0,0,0],[0,i*(self.cell_size+self.line_size)],[pic_size,i*(self.cell_size+self.line_size)])
-            return backgroud
-
-    def __load_map(self,map_path,map_data,reverse = False):
-        '''
-        Function:
-        ---------
-            读取map数据存于self.map_data，数据为0-1矩阵，位置同栅格图节点位置，默认左上角为起点，右下角为终点。
-        
-        Params:
-        -------
-            None
-        
-        Return:
-        -------
-            None
-        '''
-        if map_data == None:
-            with open(map_path,'r') as f:
-                self.map_data = f.readlines()
-            for k,i in enumerate(self.map_data):
-                self.map_data[k]= i.strip('\n')
-        else:
-            self.map_data = []
-            for  i in map_data:
-                i = [str(j) for j in i]
-                self.map_data.append(''.join(i))
-        if reverse == True:
-            self.map_data.reverse()
-        self.grid_size = len(self.map_data[0])
-    def __draw_barrier(self):
-        for i in range(len(self.map_data[0])):
-            for j in range(len(self.map_data[0])):
-                if self.map_data[i][j] == '1':
-                    x_1 = (j+1)*self.line_size + j*self.cell_size
-                    y_1 = (i+1)*self.line_size + i*self.cell_size
-                    py.draw.rect(self.pic_backgroud,[0,0,0],[x_1,y_1,self.cell_size,self.cell_size])
-
-    def draw_way(self,way_data,_2D, y_x_change = False,new_pic = True,color = [0,0,0],line_type = '-'):
-        '''
-        '_2D':'标志输入量是否为[x,y]格式',
-        'x_y_change': '调换x——y坐标',
-        'new_pic' : '新建一个背景画线段？默认未True',
-        'color' : '线段的颜色'
-        '''
-        if new_pic == True:
-            self.pic_shangetu = self.pic_backgroud.copy()
-        # 转换成二维坐标格式
-        if _2D == False:
-            way_data_1 = []
-            for i in way_data:
-                way_data_1.append([i%self.grid_size,i//self.grid_size])
-        else:
-            way_data_1 = way_data
-        if y_x_change == True:
-            for i in range(len(way_data_1)):
-                way_data_1[i][0],way_data_1[i][1] =  way_data_1[i][1],way_data_1[i][0]
-        # 画线喽
-        for k,i in enumerate(way_data_1):
-            try:
-                j = way_data_1[k+1]
-            except:
-                return None
-            point_1_x = (i[0]+1)*self.line_size + i[0]*self.cell_size+self.cell_size/2
-            point_1_y = (i[1]+1)*self.line_size + i[1]*self.cell_size+self.cell_size/2
-            point_2_x = (j[0]+1)*self.line_size + j[0]*self.cell_size+self.cell_size/2
-            point_2_y = (j[1]+1)*self.line_size + j[1]*self.cell_size+self.cell_size/2
-            # 下面两行起到上下翻转的目的
-            #point_1_y = self.backgroud_size - point_1_y
-            #point_2_y = self.backgroud_size - point_2_y
-            if line_type == '-':
-                py.draw.line(self.pic_shangetu,color,[point_1_x,point_1_y],[point_2_x,point_2_y],2)
-            elif line_type == '--': 
-                ipg.dot_line(self.pic_shangetu,color,[point_1_x,point_1_y],[point_2_x,point_2_y],2)
-                
-    def save(self,filename = '栅格图.jpg',reverse = False):
-        '''
-            Function:
-            ---------
-                将画好的栅格图存储起来。
-
-            Params:
-            -------
-                文件存放路径(含文件名)
-        '''
-        
-        try:
-            if  reverse == True:
-                self.pic_shangetu = flip(self.pic_shangetu,False,True)
-            py.image.save(self.pic_shangetu,filename)
-        except:
-            if  reverse == True:
-                self.pic_backgroud = flip(self.pic_backgroud,False,True)
-            py.image.save(self.pic_backgroud,filename)
-    def __seek_corner_point(self,way_data = [1,2,3,4]):
-        '''
-            Function:
-            ---------
-                找到所有的变异点，并返回。
-            
-            Params:
-            -------
-                way_data : list
-                    路径的节点列表
-
-            Return:
-            -------
-                special_pot_list : list
-                    可疑节点信息列表，元素为数值则为转折点，否则若为size=2列表，则为擦边点。
-                special_pot_key_list : list
-                    节点标记信息，元素为1，2 
-        '''
-        #Step 1:找到转折点，以及擦边点，统称变异点，转折点用1标记，擦边点用2标记
-        a_1 = way_data[1]-way_data[0]    # 通过分析两节点之间的数值差的变化，来得到转折点信息。
-        special_pot_list = [0]                #存放变异点的列表,初始添加起点
-        special_pot_key_list = [1]
-        for i in range(len(way_data)-1):
-            if way_data[i+1]-way_data[i] != a_1:   # 数值差不一样了
-                special_pot_list.append(way_data[i])
-                special_pot_key_list.append(1)
-                a_1 = way_data[i+1]-way_data[i]
-                if self.__is_near_obstacle(way_data[i],way_data[i+1]):
-                    special_pot_list.append([way_data[i],way_data[i+1]])
-                    special_pot_key_list.append(2)
-        special_pot_list.append((self.grid_size**2)-1)
-        special_pot_key_list.append(1)
-        print(special_pot_list)
-        print(special_pot_key_list)
-        return special_pot_list, special_pot_key_list
-
-    def __is_near_obstacle(self,point_a,point_b):
-        '''
-            Function:
-            ---------
-                用来判定某段路径是否擦肩而过障碍物，该函数仅被设计供sek_corner_point方法使用。
-            
-            Params:
-            -------
-                point_a:    int
-                    起始节点
-                point_b:    int
-                    终止节点
-            
-            Return:
-            -------
-                result：True or False
-        '''
-        point_a_x,point_a_y = point_a % self.grid_size, point_a // self.grid_size
-        point_b_x,point_b_y = point_b % self.grid_size, point_b // self.grid_size
-        if (point_a_x==point_b_x) or (point_a_y == point_b_y):  # 同行或同列直接略过
-            return False
-        elif (point_b_x == point_a_x + 1):   #水平方向向右(下面的代码中关于y坐标的运算中使用了“self.grid_size-1”，因为列表序数方向是从上往下，而我们对栅格图的y方向定为自下而上)
-            if (point_b_y == point_a_y + 1):  #垂直方向向下
-                if (self.map_data[self.grid_size-1-(point_a_y+1)][point_a_x] == '1') or(self.map_data[self.grid_size-1-point_a_y][point_a_x+1] == '1'):
-                    return True
-                else:
-                    return False
-            else:                             #垂直向上
-                if (self.map_data[self.grid_size-1-point_a_y-1][point_a_x] == '1') or(self.map_data[self.grid_size-1-point_a_y][point_a_x+1] == '1'):
-                    return True
-                else:
-                    return False
-        elif (point_b_x == point_a_x - 1):   #水平方向向左
-            if (point_b_y == point_a_y + 1):  #垂直方向向下
-                if (self.map_data[self.grid_size-1-(point_a_y+1)][point_a_x] == '1') or(self.map_data[self.grid_size-1-point_a_y][point_a_x-1] == '1'):
-                    return True
-                else:
-                    return False
-            else:                             #垂直向上
-                if (self.map_data[self.grid_size-1-(point_a_y-1)][point_a_x] == '1') or(self.map_data[self.grid_size-1-point_a_y][point_a_x-1] == '1'):
-                    return True
-                else:
-                    return False
-
-    def __calc_abs_x_y(self,point_data = 0):
-        '''
-            Function:
-            ---------
-                计算转折点或擦边点的像素坐标。
-            
-            Params:
-                point_data : int or list
-                    int时为转折点，list时为擦边点
-
-            Return:
-                [x,y] : [int,int]
-                    像素坐标值
-        '''
-        if type(point_data) == int:
-            x,y = point_data%self.grid_size,point_data//self.grid_size
-            x = x*self.cell_size + (x+1)*self.line_size + self.cell_size//2+1
-            y = y*self.cell_size + (y+1)*self.line_size + self.cell_size//2+1
-            return x,y
-        elif type(point_data) == list:
-            x_1,y_1 = point_data[0]%self.grid_size,point_data[0]//self.grid_size
-            x_1 = x_1*self.cell_size + (x_1+1)*self.line_size + self.cell_size//2+1
-            y_1 = y_1*self.cell_size + (y_1+1)*self.line_size + self.cell_size//2+1
-
-            x_2,y_2 = point_data[1]%self.grid_size,point_data[1]//self.grid_size
-            x_2 = x_2*self.cell_size + (x_2+1)*self.line_size + self.cell_size//2+1
-            y_2 = y_2*self.cell_size + (y_2+1)*self.line_size + self.cell_size//2+1
-
-            x = abs(x_1 - x_2)//2 + min(x_1,x_2)
-            y = abs(y_1 - y_2)//2 + min(y_1,y_2)
-            return x,y
-
-    def opti_way(self, way_data = [1,2,3,4]):
-        #Step 1:找到转折点，以及擦边点，统称变异点
-        a_1,b_1 = self.__seek_corner_point(way_data=way_data)
-        #Step 2:计算其绝对像素坐标，并打印出来看看
-        print('a_1：',a_1)
-        print('b_1：',b_1)
-
-        abs_xy_list = []
-        try:
-            for i in a_1:
-                x,y = self.__calc_abs_x_y(point_data=i)
-                py.draw.circle(self.pic_shangetu,[50,100,200],[x,self.backgroud_size - y],3)
-                abs_xy_list.append([x,y])
-        except:
-            self.pic_shangetu = self.pic_backgroud.copy()
-            for i in a_1:
-                x,y = self.__calc_abs_x_y(point_data=i)
-                abs_xy_list.append([x,y])
-            #self.draw_pot(x,y)
-        # Step 3: 优化
+    def run(self):
         running = True
-        i = 0
         while running:
-            if b_1[i+1] == 2:
-                print(abs_xy_list[i],abs_xy_list[i+1],abs_xy_list[i+2],'中间点为擦边点！')
-                pass    
-            else:
-                result = self.__trigon_opt(abs_xy_list[i],abs_xy_list[i+1],abs_xy_list[i+2],b_1[i:i+3])
-                if result == None:
-                    print(abs_xy_list[i],abs_xy_list[i+1],abs_xy_list[i+2],'能直接连接！')
-                    del abs_xy_list[i+1]
-                    del b_1[i+1]
-                    i -= 1
-                else:
-                    print(abs_xy_list[i],abs_xy_list[i+1],abs_xy_list[i+2],'不能直接连接！,添加新节点')
-                    if result in abs_xy_list:
-                        pass
-                    else:
-                        abs_xy_list = abs_xy_list[0:i+1]+ [result]+abs_xy_list[i+1:]
-                        b_1 = b_1[0:i+1] + [1] + b_1[i+1:]
-                    
+            #step 1: select an element whose f is smallest(if there are multiple elements with the same f value, we will select the one with the lowest s value.)
+            smal_index = self.open_list.loc[:,'f'].argmin()
+            #step 2: find all the neighbor grid of it
+            neighbour_allowed = self._find_neighbour(self.open_list.iloc[smal_index].values)
+            #step 3: update open_list by neighbour_allowed
+            have_find_term = self._update_open_list(neighbour_allowed,self.open_list.iloc[smal_index].values)
+            if have_find_term == True:
+                break
+            #step 4: delete it in the open_list and copy to the close_list
+            self.close_list.loc[self.close_list.shape[0]] = self.open_list.iloc[smal_index].copy()
+            self.open_list.drop(smal_index,inplace=True)
+            self.open_list.index = range(self.open_list.shape[0])
+        self._calc_way()
+        #!print('最优路径为:')
+        #!(self.best_way_data)
+        
 
-            i += 1
-            if i >= len(abs_xy_list)-2:
-                break  
-        print('优化后的abs_xy_list:',abs_xy_list)
-        # Step 4: 画出优化的线
-        self.final_opt_way = abs_xy_list
-        for i in abs_xy_list:
-            x,y = i[0],i[1]
-            py.draw.circle(self.pic_shangetu,[255,150,50],[x,self.backgroud_size - y],3)
-        for i in range(len(abs_xy_list)-1):
-            py.draw.line(self.pic_shangetu,[255,150,50],[abs_xy_list[i][0],self.backgroud_size-abs_xy_list[i][1]],[abs_xy_list[i+1][0],self.backgroud_size-abs_xy_list[i+1][1]],3)
 
-        #self.save('栅格图优化.jpg')
-        #       
-    def __trigon_opt(self,pot_1,pot_2,pot_3,key_list):
-        '''
-            Function:
-            --------
-                三角形优化，形象的来看，就是把pot_1, pot_2, pot_3组成的三角形尽可能地变得扁。
-            
-            Params:
-            -------
-                pot_1 : [int,int]
-                    第一点的坐标，pot_2 ,pot_3同
-                key_list : list , size = 3
-                    三个点的类型标记
+              
 
-            Return:
-            -------
-                pot: [int,int] or None
-                若pot为None，则代表可疑直连，否则pot为三角的极限顶点。
-        '''
-
-        result = self.__is_accross_barrier(pot_1,pot_3)
-        if result[0] == True:
-            pot = self.__deep_opt(pot_1,pot_2,pot_3)
-            #input('wait for ldd...')
-            return pot
-
-        else:
-            return None
-
-    def __is_accross_barrier(self,pot_1 ,pot_2):
+    def _find_neighbour(self,who):
         '''
             Function:
             ---------
-                判断两点之间有否有障碍物存在。微元法，步进尺寸为L/50
-        '''
-        #if pot_1 == [321,449]:
-        #    print('1')
-        deta_x = (pot_2[0]-pot_1[0])/50
-        deta_y = (pot_2[1]-pot_1[1])/50
-        for i in range(1,51):      #步进100个点判断
-            x = pot_1[0]+int(deta_x*i)
-            y = pot_1[1]+int(deta_y*i)
-            if (x-self.line_size)%(self.cell_size+self.line_size) == 0:  #点正好在直线上，故不用考虑这个点了
-                continue
-            else:                                      #若不在直线上，则需判断它在哪列
-                for i_1 in range(0,self.grid_size):
-                    a_1 = i_1*self.cell_size+(i_1+1)*self.line_size
-                    if a_1<x<=a_1+self.cell_size:
-                        x_1 = i_1
-                        break
-            if (y-self.line_size)%(self.cell_size+self.line_size) == 0:  #点正好在直线上，故不用考虑这个点了
-                continue
-            else:                                      #若不在直线上，则需判断它在哪列
-                for i_1 in range(0,self.grid_size):
-                    a_1 = i_1*self.cell_size+(i_1+1)*self.line_size
-                    if a_1<y<=a_1+self.cell_size:
-                        y_1 = i_1
-                        break 
-            if self.map_data[self.grid_size-1-y_1][x_1] == '1':         #判断该格子上是否障碍物，是的话就返回True
-                print('判断节点:',x_1,y_1,'结果不可通行')
-                return [True,[x_1,y_1]]
-            #else:
-            #    print('判断节点:',self.grid_size-1-y_1,x_1,'结果为可通行')
-        return [False,[]]                                #100个点都判断完了，返回False
-     
-    def __deep_opt(self,pot_1,pot_2,pot_3):
-        '''
-        function:
-        ---------
-        深度三角优化，找到越过障碍物的临界状态，并返回新的中间点。
+            This Func can find all neighbor grid of the who, and return them.
 
-        Params:
-            pot_1,pot_2,pot_3 : [int,int]
-                起点，中间点、终点
+            Params:
+            -------
+            who : 1*d-list--> fe:[[0,0],0,19,19,-1]
 
-        Return:
-            pot : [int, int]
+            Return:
+            ---------
+            neighbour_allowed : 1*d-list--> all the neighbour can allowed.
         '''
-        print(pot_1,pot_2,pot_3)
-        pot_2_1 = [(pot_1[0]+pot_3[0])//2,(pot_1[1]+pot_3[1])//2]
-        #pot_1[1] = self.backgroud_size - pot_1[1]
-        #pot_2[1] = self.backgroud_size - pot_2[1]
-        #pot_2_1[1] = self.backgroud_size - pot_2_1[1]
-        #pot_3[1] = self.backgroud_size - pot_3[1]
-        #py.draw.line(self.pic_shangetu,[0,0,255],pot_1,pot_3)
-        #py.draw.line(self.pic_shangetu,[0,0,255],pot_2,pot_3)
-        #py.draw.line(self.pic_shangetu,[0,0,255],pot_1,pot_2)
-        #py.draw.line(self.pic_shangetu,[0,0,255],pot_2,pot_2_1)
-        #py.draw.circle(self.pic_shangetu,[0,0,255],pot_2_1,3)
-        STEP = 50
-        deta_x = (pot_2_1[0]-pot_2[0])/STEP        #20步微元法找临界状态
-        deta_y = (pot_2_1[1]-pot_2[1])/STEP
-        for i in range(1,STEP+1):
-            x_1 = pot_2[0] + int(deta_x*i)
-            y_1 = pot_2[1] + int(deta_y*i)
-            r_1 = self.__is_accross_barrier(pot_1,[x_1,y_1])
-            r_2 = self.__is_accross_barrier([x_1,y_1],pot_3)
-            if r_1[0]:
-                #print('找到临界障碍物:',r_1[1])
-                new_pot_2 = self.__seek_shortest_vertice(r_1[1],pot_1,[x_1,y_1],pot_3)
-                py.draw.circle(self.pic_shangetu,[255,0,0],[new_pot_2[0],self.backgroud_size - new_pot_2[1]],3)
-                #self.save()
-                #input('wait...')
-                return new_pot_2
-            elif r_2[0]:
-                #print('找到临界障碍物:',r_2[1])
-                new_pot_2 = self.__seek_shortest_vertice(r_2[1],pot_1,[x_1,y_1],pot_3)
-                py.draw.circle(self.pic_shangetu,[255,0,0],[new_pot_2[0],self.backgroud_size - new_pot_2[1]],3)
-                #self.save()
-                #input('wait...')
-                return new_pot_2
+        y = int(who[0])
+        x = int(who[1])
+        neighbour_1 = [
+            [y-1,x-1],[y-1,x],[y-1,x+1],
+            [y  ,x-1],        [y  ,x+1],
+            [y+1,x-1],[y+1,x],[y+1,x+1],
+        ]
+        neighbour_allowed = []
+        for i in neighbour_1:
+            if (0<=i[0]<=self.size_map-1)and(0<=i[1]<=self.size_map-1): #grid within the scope of the map.
+                if self.map_data[i[0],i[1]] == 0:
+                    neighbour_allowed.append(i)
+        return neighbour_allowed
+
+    def _update_open_list(self,neighbour_allowed,who):
+        have_find_term = False           # maiks whether the terminal was found
+        for i in neighbour_allowed:
+            # case 1: if i in close_list, discard it. 
+            
+            if (i == self.close_list.loc[:,['pos_y','pos_x']].values).all(axis = 1).any():
+                continue
+            # we must calculate the data first as it will be used in the both following case. 
+            dict_1 = {
+                'pos_y' : i[0],
+                'pos_x' : i[1],
+                'g'   : who[2]+(1.414 if (abs(i[0]-who[0])+abs(i[1]-who[1]))==2 else 1),
+                'h'   : abs(i[0]-self.end[0])+abs(i[1]-self.end[1]),
+                'f'   : 0,
+                'parent_y': who[0],
+                'parent_x': who[1]
+            }
+            dict_1['f']  = dict_1['g']+dict_1['h']
+            # case 2: provided i in open_list, check whether i's f is greater 
+            #              than b's, if True, update the data of i in the open_list;
+            #              otherwise, discard new it. 
+            r = (i == self.open_list.loc[:,['pos_y','pos_x']].values).all(axis = 1)
+            if r.any():
+                index = np.where(r==True)[0]
+                if self.open_list.loc[index,'f'].values>dict_1['f']:
+                    self.open_list.loc[index,'g'] = dict_1['g']
+                    self.open_list.loc[index,'h'] = dict_1['h']
+                    self.open_list.loc[index,'f'] = dict_1['f']
+                    self.open_list.loc[index,'parent_y'] = dict_1['parent_y']
+                    self.open_list.loc[index,'parent_x'] = dict_1['parent_x']
+                    
+                else:
+                    pass
+            # case 3: add the i in the open_list.(when i is terminal,the return_param will be True)
             else:
-                pass
-                #print('继续...')
-        #self.save()
-        
-    def __seek_shortest_vertice(self,grid,pot_1,pot_2,pot_3):
-        # Step 1: 先找到四个顶点坐标
-        x = grid[0]
-        y = grid[1]
-        vertices = []
-        vertices.append([x*self.cell_size+(x+1)*self.line_size,y*self.cell_size+(y+1)*self.line_size])  #左下角点
-        vertices.append([x*self.cell_size+(x+1)*self.line_size,(y+1)*self.cell_size+(y+2)*self.line_size])  #左上角点
-        vertices.append([(x+1)*self.cell_size+(x+2)*self.line_size,y*self.cell_size+(y+1)*self.line_size])  #右下角点
-        vertices.append([(x+1)*self.cell_size+(x+2)*self.line_size,(y+1)*self.cell_size+(y+2)*self.line_size])  #左下角点
-        yuzhi = 99999 
-        key = 0
-        v1 = [pot_2[0]-pot_1[0],pot_2[1]-pot_1[1]]
-        v2 = [pot_2[0]-pot_3[0],pot_2[1]-pot_3[1]]
-        for k,i in  enumerate(vertices):
-            v1_1 = [i[0]-pot_1[0],i[1]-pot_1[1]]
-            v2_1 = [i[0]-pot_3[0],i[1]-pot_3[1]]
-            s_1 = abs(v1_1[0]*v1[1]-v1_1[1]*v1[0])
-            s_2 = abs(v2_1[0]*v2[1]-v2_1[1]*v2[0])
-            print(k,s_1,s_2)
-            s = min(s_1,s_2)
-            if s<yuzhi:
-                key = k
-                yuzhi = s
-        return vertices[key]
-     
-    # def calc_way_lenght(self):
-    #     self.way_lenght = 0
-    #     for i in range(len(self.final_opt_way)-1):
-    #         a_1 = self.final_opt_way[i]
-    #         b_1 = self.final_opt_way[i+1]
-    #         self.way_lenght += (((a_1[0]-b_1[0])**2)+((a_1[1]-b_1[1])**2))**0.5
-    #     self.way_lenght = self.way_lenght/self.cell_size
-        
-        
+                self.open_list.loc[self.open_list.shape[0]] = dict_1
+            if (i == self.end).all():
+                have_find_term = True
+        return have_find_term
 
+    def _calc_way(self):
+        '''
+            Function:
+            ---------
+                find the best way from open_list and close_list, and the result will be saved in self.best_way_data.
+        '''
+        #step 1: Ectract the position and parent of each element in open_list and close_list.
+        dict_1 = {
 
+        }
+        for k in range(self.open_list.shape[0]):
+            i = self.open_list.iloc[k]
+            dict_1[str(int(i.values[0]))+','+str(int(i.values[1]))] = str(int(i.values[-2]))+','+str(int(i.values[-1]))
+        for k in range(self.close_list.shape[0]):
+            i = self.close_list.iloc[k]
+            dict_1[str(int(i.values[0]))+','+str(int(i.values[1]))] = str(int(i.values[-2]))+','+str(int(i.values[-1]))
+        # step 2: find the best way between the end with start from dict_1.
+        y,x = self.end
+        self.best_way_data = [[y,x]]
+        while True:
+            a_1 = dict_1[str(y)+','+str(x)].split(',')
+            y,x = eval(a_1[0]),eval(a_1[1])
+            b_1 = self.best_way_data[-1]
+            self.best_way_len += ((y-b_1[0])**2+(x-b_1[1])**2)**0.5
+            self.best_way_data.append([y,x])
+            if y==self.start[0] and x == self.start[1]:
+                break
+        self.best_way_data.reverse()
+            
 
 
 
