@@ -6,11 +6,33 @@ class Map():
     def __init__(self) -> None:
         pass
         self.data = []    # map中的数据
+
+    def load_map_file(self,fp):
+        '''
+        从map文件中读取地图数据。
+        '''
+        with open(fp,'r') as f:
+                a_1 = f.readlines()
+        for i in range(len(a_1)):
+            a_1[i] = a_1[i].strip('\n')
+        for i in a_1:
+            l = []
+            for j in i:
+                if j == '0':
+                    l.append(0)
+                else:
+                    l.append(1)
+            self.data.append(l)
+        
+
     def map_reverse(self,fp='map.dll'):
+        '''
+        自动读取map地图文件，并翻转数据再储存于原文件中。
+        '''
         with open(fp,'r') as f:
             data = f.readlines()
         data.reverse()
-        with open('map2.dll','w') as f:
+        with open(fp,'w') as f:
             f.writelines(data)
 
     def recognition(self,fig_path,size):
@@ -74,8 +96,8 @@ class Map():
                 if i.type == pg.QUIT:
                     running = False
                 elif i.type == pg.MOUSEBUTTONDOWN:
-                    self.change(i.pos,screen)
-    def change(self,i_pos,screen):
+                    self._change(i.pos,screen)
+    def _change(self,i_pos,screen):
         print('修改行为:',end=' ')
         dis = 9999
         best_pos = []
